// MIDIConverter stores standard MIDI-files as ASCII-TXT-files and vice versa
// Copyright (C) 2000 Michael Bliem      email: e9271117@student.tuwien.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import java.io.*;
import java.util.*;

import MIDI;
import MIDIUtil;
import MIDIFrame;

public class MIDIConverter
{
	static MIDI openMIDI(String s)
	// returns a MIDI-instance for the File named s (or null if fails)
	{
		MIDI m = null;
		FileInputStream fpp = null;
		try { fpp = new FileInputStream(new File(s)); }
		catch(IOException e) { System.out.println("File \'"+s+"\' not found."); return null; }
		try {
			String header = MIDIUtil.getString(4,fpp);
			if(header.equals("RIFF") || header.equals("MThd")) MIDIUtil.setFileType(true);
			else MIDIUtil.setFileType(false);
			fpp.close();
			fpp = new FileInputStream(new File(s));
			m = new MIDI(fpp); }
		catch(IOException e) { System.out.println("Could not read file \'"+s+"\'."); }
		catch(MIDIException e) { System.out.println("Error reading file \'"+s+"\':\n"+e); }
		finally {
			try {	fpp.close(); return m; }
			catch(IOException e) {
				System.out.println("Could not close file \'"+s+"\'.");
				return m;
			}
		}
	}

	static boolean saveMIDI(MIDI m,boolean toMidi,String s)
	// writes MIDI m to the File named s (toMIDI if .mid)
	{
		FileOutputStream fpp = null;
		try { fpp = new FileOutputStream(new File(s)); }
		catch(IOException e) {
			System.out.println("Could not create file \'"+s+"\'."); return false; }
		try {
			if(toMidi) m.toMIDI(fpp); else m.toTXT(fpp);
			fpp.close();
			return true; }
		catch(IOException e) {
			System.out.println("Could not close file \'"+s+"\'."); return false; }
		catch(MIDIException e) {
			System.out.println(e+"Could not write file \'"+s+"\'.");
			try {	fpp.close(); return false; }
			catch(IOException f) {
				System.out.println("Could not close file \'"+s+"\'."); return false;
			}
		}
	}

	static void winMain()
	// runs MIDIConverter in WindowsVersion
	{ MIDIFrame f = new MIDIFrame(); }

	static void printUsage()
	// prints usage of MIDIConverter to console
	{
		System.out.print("\nUSAGE: MIDIConverter ");
		System.out.println("[-h|-l|ReadFile {-d delFile|-i insFile|-r type} WriteFile]\n");
		System.out.println("  -h:             Prints this help.");
		System.out.println("  -l:             Prints license information.");
		System.out.println("  ReadFile:       SourceFile with main MIDI-data.");
		System.out.println("  WriteFile:      Saved as MIDI-file if \'.mid\'; ASCII otherwise.");
		System.out.println("  -d delFile:     Deletes delFile from SourceFile.");
		System.out.println("  -i insFile:     Inserts insFile to SourceFile.");
		System.out.println("  -r type:        Removes data from SourceFile specified by type:");
		System.out.println("                  nt:          NoteOn and NoteOff");
		System.out.println("                  pr:          ChannelPressure");
		System.out.println("                  at:          AfterTouch");
		System.out.println("                  pb:          PitchBend");
		System.out.println("                  cc:          ControlChange");
		System.out.println("                  cc0...cc127: ControlChange number");
		System.out.println("                  pc:          ProgramChange");
		System.out.println("                  sy:          SystemMessages");
		System.out.println("                  mt:          MetaEvents.\n");
		System.out.println("Use without parameters to start windows version.");
	}

	public static void main(String args[])
	// reads and handles concernig to args
	{
		if(args.length == 0) { winMain(); return; }												// Windows Version
		else if((args.length > 0) && (args[0].equals("-l"))) {						// Print License
			System.out.println("MIDIConverter version 1.42, Copyright (C) 2000 Michael Bliem");
			System.out.println("MIDIConverter comes with ABSOLUTELY NO WARRANTY;");
			System.out.println("for details read the GNU General Public License.");
			System.out.println("This is free software, and you are welcome to redistribute it");
			System.out.println("under certain conditions;");
			System.out.println("for details read the GNU General Public License.\n");
			return; }
		else if((args.length < 2) || ((args.length % 2) != 0)) { printUsage(); return; }
		MIDI main = openMIDI(args[0]);
		if(main == null) return;
		for(int i = 1;i < args.length - 1;i+=2) {
			if(args[i].equals("-d")) main.deleteMIDI(openMIDI(args[i+1]));					// deleting
			else if(args[i].equals("-i")) main.insertMIDI(openMIDI(args[i+1]));			// inserting
			else if(args[i].equals("-r")) {																					// removing:
				if(args[i+1].equals("nt")) {																					// notes
					main.deleteEvent(-1,-1,0x80,-1,null);
					main.deleteEvent(-1,-1,0x90,-1,null); }
				else if(args[i+1].equals("pr")) main.deleteEvent(-1,-1,0xd0,-1,null);	// pressure
				else if(args[i+1].equals("at")) main.deleteEvent(-1,-1,0xa0,-1,null);	// touch
				else if(args[i+1].equals("pb")) main.deleteEvent(-1,-1,0xe0,-1,null); // pitchbend
				else if(args[i+1].equals("cc")) main.deleteEvent(-1,-1,0xb0,-1,null);	// control
				else if(args[i+1].equals("pc")) main.deleteEvent(-1,-1,0xc0,-1,null);	// program
				else if(args[i+1].equals("mt")) main.deleteEvent(-1,-1,0xf0,0x0f,null);	// meta
				else if(args[i+1].equals("sy")) {																			// system
					main.deleteEvent(-1,-1,0xf0,0x07,null);
					main.deleteEvent(-1,-1,0xf0,0x00,null); }
				else if(args[i+1].startsWith("cc")) {																	// controlNr.
					byte[] data = {(byte)new Integer(args[i+1].substring(2)).intValue(),-1};
					main.deleteEvent(-1,-1,0xb0,-1,data); }
				else { printUsage(); return; }}
			else { printUsage(); return; }
		}
		String s = args[args.length - 1];
		if(((s.length() > 3) && s.substring(s.length()-4).equalsIgnoreCase(".MID")) ||
			 ((s.length() > 4) && s.substring(s.length()-5).equalsIgnoreCase(".MIDI")))
			saveMIDI(main,true,s);
		else saveMIDI(main,false,s);
	}
}