// MIDIConverter stores standard MIDI-files as ASCII-TXT-files and vice versa
// Copyright (C) 2000 Michael Bliem      email: e9271117@student.tuwien.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import java.io.*;
import java.awt.*;
import java.awt.event.*;

import MIDI;
import MIDIUtil;

class MIDIFrame
// MIDIConverters GUI; Window and EventHandling with Inner Classes (see API 1.1 docs)
{
	static MIDI main = null;				// main MIDI-file
	static Frame mFrame;						// the window
	static TextArea mText;					// the text-field
	static Dialog mDialog;					// dialog for handling remove
	static String fileName = null;	// last used main-MIDI filename
	static String dirName = null;		// last used directory
	static boolean german;					// marks German Notation
	static boolean comments;				// marks Comments or not
	static int barOffset;						// bar starts counting with
	static int programOffset;				// progNumber starts with

	MIDIFrame()
	{
		mFrame = new Frame("MIDI Converter 2.0");						// Window (with name)
		mFrame.addWindowListener(new WindowListener() {			// inner class for event handling
			 public void windowOpened(WindowEvent e) {}
			 public void windowClosing(WindowEvent e) { mFrame.dispose(); System.exit(0); }
			 public void windowClosed(WindowEvent e) {}
			 public void windowIconified(WindowEvent e) {}
			 public void windowDeiconified(WindowEvent e) {}
			 public void windowActivated(WindowEvent e) {}
			 public void windowDeactivated(WindowEvent e) {}
			});
		MenuBar mBar;													// the MenuBar
		Menu m;
		MenuItem mItem;
		String bool;													// used for boolean switching
		german = MIDIUtil.GERMAN;							// inits option values
		comments = MIDIUtil.COMMENTS;
		barOffset = MIDIUtil.BAROFFSET;
		programOffset = MIDIUtil.PROGRAMOFFSET;
		mFrame.add(mText = new TextArea());		// output text-area
		mFrame.setMenuBar(mBar = new MenuBar());

		mText.append("MIDIConverter version 2.0, Copyright (C) 2000 Michael Bliem\n");
		mText.append("MIDIConverter comes with ABSOLUTELY NO WARRANTY;\n");
		mText.append("for details read the GNU General Public License.\n");
		mText.append("This is free software, and you are welcome to redistribute it\n");
		mText.append("under certain conditions;\n");
		mText.append("for details read the GNU General Public License.\n\n");


//  -------------------------------------- FILE ------------------------------------------

		mBar.add(m = new Menu("File"));


		m.add(mItem = new MenuItem("Open...",new MenuShortcut(79)));					// O for Open
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				FileDialog fd = new FileDialog(mFrame,"Open file...",FileDialog.LOAD);
				if(dirName != null) fd.setDirectory(dirName);
				fd.show();
				String newName;
				dirName = fd.getDirectory();
				newName = fd.getFile();
				if((dirName != null) && (newName != null)) newName = dirName.concat(newName);
				if(newName != null) {
					fileName = newName;
					mText.append("Open \'"+fileName+"\'...");
					main = MIDIConverter.openMIDI(fileName);
					if(main == null) {
						mText.append("failed.\nLook at console for details.\n");
						mFrame.setTitle("MIDI Converter 2.0"); }
					else { mText.append("done.\n"); mFrame.setTitle(fileName); }}
				MIDIUtil.setProgramOffset(programOffset);
				MIDIUtil.setBarOffset(barOffset); }});

		m.add(mItem = new MenuItem("Save TXT",new MenuShortcut(65)));					// A for SaveTXT
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if((fileName == null) || (main == null))
					mText.append("Open a file before saving.\n");
				else {
					String newName;
					if(((fileName.length() > 3) &&
							fileName.substring(fileName.length()-4).equalsIgnoreCase(".MID")) ||
						 ((fileName.length() > 4) &&
						 	fileName.substring(fileName.length()-5).equalsIgnoreCase(".MIDI")))
						newName = fileName.concat(".txt");
					else newName = fileName;
					mText.append("Save \'"+newName+"\' as TXT...");
					if(MIDIConverter.saveMIDI(main,false,newName)) mText.append("done.\n\n");
					else mText.append("failed.\nLook at console for details.\n"); }
				}});

		m.add(mItem = new MenuItem("Save MIDI",new MenuShortcut(83)));				// S for SaveMIDI
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if((fileName == null) || (main == null))
					mText.append("Open a file before saving.\n");
				else {
					String newName;
					if(((fileName.length() > 3) &&
							fileName.substring(fileName.length()-4).equalsIgnoreCase(".MID")) ||
						 ((fileName.length() > 4) &&
						 	fileName.substring(fileName.length()-5).equalsIgnoreCase(".MIDI")))
						newName = fileName;
					else newName = fileName.concat(".mid");
					mText.append("Save \'"+newName+"\' as MIDI...");
					if(MIDIConverter.saveMIDI(main,true,newName)) mText.append("done.\n\n");
					else mText.append("failed.\nLook at console for details.\n"); }
				}});

		m.add(mItem = new MenuItem("Save TXT as...",new MenuShortcut(84)));		// T TXT save As
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(main == null) mText.append("Open a file before saving.\n");
				else {
					FileDialog fd = new FileDialog(mFrame,"Save TXT-file as...",FileDialog.SAVE);
					if(dirName != null) fd.setDirectory(dirName);
					fd.show();
					String newName;
					dirName = fd.getDirectory();
					newName = fd.getFile();
					if((dirName != null) && (newName != null)) newName = dirName.concat(newName);
					if(newName != null) {
						fileName = newName;
						mText.append("Save \'"+fileName+"\' as TXT...");
						if(MIDIConverter.saveMIDI(main,false,fileName)) {
							mText.append("done.\n\n");
							mFrame.setTitle(fileName); }
						else mText.append("failed.\nLook at console for details.\n");
					}}
				}});

		m.add(mItem = new MenuItem("Save MIDI as...",new MenuShortcut(77)));	// M MIDI save As
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(main == null) mText.append("Open a file before saving.\n");
				else {
					FileDialog fd = new FileDialog(mFrame,"Save MIDI-file as...",FileDialog.SAVE);
					if(dirName != null) fd.setDirectory(dirName);
					fd.show();
					String newName;
					dirName = fd.getDirectory();
					newName = fd.getFile();
					if((dirName != null) && (newName != null)) newName = dirName.concat(newName);
					if(newName != null) {
						fileName = newName;
						mText.append("Save \'"+fileName+"\' as MIDI...");
						if(MIDIConverter.saveMIDI(main,true,fileName)) {
							mText.append("done.\n\n");
							mFrame.setTitle(fileName); }
						else mText.append("failed.\nLook at console for details.\n");
					}}
				}});

		m.addSeparator();
		m.add(mItem = new MenuItem("Quit",new MenuShortcut(81)));							// Q for Quit
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { mFrame.dispose(); System.exit(0); }});


//  -------------------------------------- EDIT ------------------------------------------

		mBar.add(m = new Menu("Edit"));


		m.add(mItem = new MenuItem("Insert...",new MenuShortcut(73)));				// I for Insert
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(main == null) mText.append("Open a file before inserting.\n");
				else {
					FileDialog fd = new FileDialog(mFrame,"Open insert-file...",FileDialog.LOAD);
					if(dirName != null) fd.setDirectory(dirName);
					fd.show();
					String newName;
					dirName = fd.getDirectory();
					newName = fd.getFile();
					if((dirName != null) && (newName != null)) newName = dirName.concat(newName);
					if(newName != null) {
						mText.append("Insert \'"+newName+"\'...");
						MIDI ins = MIDIConverter.openMIDI(newName);
						if(ins == null) mText.append("failed.\nLook at console for details.\n");
						else { main.insertMIDI(ins); mText.append("done.\n"); }
					}}
				MIDIUtil.setProgramOffset(programOffset);
				MIDIUtil.setBarOffset(barOffset); }});

		m.add(mItem = new MenuItem("Delete...",new MenuShortcut(68)));				// D for Delete
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(main == null) mText.append("Open a file before deleting.\n");
				else {
					FileDialog fd = new FileDialog(mFrame,"Open delete-file...",FileDialog.LOAD);
					if(dirName != null) fd.setDirectory(dirName);
					fd.show();
					String newName;
					dirName = fd.getDirectory();
					newName = fd.getFile();
					if((dirName != null) && (newName != null)) newName = dirName.concat(newName);
					if(newName != null) {
						mText.append("Delete \'"+newName+"\'...");
						MIDI del = MIDIConverter.openMIDI(newName);
						if(del == null) mText.append("failed.\nLook at console for details.\n");
						else { main.deleteMIDI(del); mText.append("done.\n"); }
					}}
				MIDIUtil.setProgramOffset(programOffset);
				MIDIUtil.setBarOffset(barOffset); }});

		m.add(mItem = new MenuItem("Remove...",new MenuShortcut(82)));				// R for Remove
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(main == null)
					mText.append("Open a file before removing.\n");
				else {
					mText.append("Remove events...");
					removeDialog();
					mText.append("done.\n"); }
				}});


//  ------------------------------------- OPTIONS ----------------------------------------

		mBar.add(m = new Menu("Options"));


		if(comments) bool = new String("on"); else bool = new String("off");
		m.add(mItem = new MenuItem("Comments "+bool,new MenuShortcut(67)));		// C for COMMENTS
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String s;
				comments = !comments;
				MIDIUtil.setComments(comments);
				if(comments) s = new String("on"); else s = new String("off");
				mFrame.getMenuBar().getMenu(2).getItem(0).setLabel("Comments "+s);
				mText.append("Comments turned "+s+".\n");
			}});

		if(german) bool = new String("on"); else bool = new String("off");
		m.add(mItem = new MenuItem("German Notation "+bool,new MenuShortcut(71)));	// G GERMAN
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String s;
				german = !german;
				MIDIUtil.setGerman(german);
				if(german) s = new String("on"); else s = new String("off");
				mFrame.getMenuBar().getMenu(2).getItem(1).setLabel("German Notation "+s);
				mText.append("German notation turned "+s+".\n");
			}});

		if(programOffset > 0) bool = new String("on"); else bool = new String("off");
		m.add(mItem = new MenuItem("Program Offset "+bool,new MenuShortcut(78)));		// N PrgNr
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String s;
				if(programOffset > 0) programOffset = 0; else programOffset = 1;
				MIDIUtil.setProgramOffset(programOffset);
				if(programOffset > 0) s = new String("on"); else s = new String("off");
				mFrame.getMenuBar().getMenu(2).getItem(2).setLabel("Program Offset "+s);
				mText.append("Program offset turned "+s+".\n");
			}});

		if(barOffset > 0) bool = new String("on"); else bool = new String("off");
		m.add(mItem = new MenuItem("Bar Offset "+bool,new MenuShortcut(66)));	// B BAROffset
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String s;
				if(barOffset > 0) barOffset = 0; else barOffset = 1;
				MIDIUtil.setBarOffset(barOffset);
				if(barOffset > 0) s = new String("on"); else s = new String("off");
				mFrame.getMenuBar().getMenu(2).getItem(3).setLabel("Bar Offset "+s);
				mText.append("Bar offset turned "+s+".\n");
			}});


//  -------------------------------------- HELP ------------------------------------------

		mBar.add(m = new Menu("Help"));


		m.add(mItem = new MenuItem("MIDI-Converter",new MenuShortcut(72)));		// H for Help
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				FileInputStream fpp = null;
				String s = new String("MIDIConverter.txt");
				try { fpp = new FileInputStream(new File(s)); }
				catch(IOException ex) {
					System.out.println("File \'" + s + "\' not found.");
					mText.append("Help...failed.\nLook at console for details.\n"); return; }
				try {
					s = MIDIUtil.getLine(fpp);
					while(s != null) { mText.append(s+"\n"); s = MIDIUtil.getLine(fpp); }
					fpp.close(); }
				catch(IOException ex) { System.out.println("Could not close file \'"+s+"\'."); }
				catch(MIDIException ex) {
					System.out.println("Error reading file \'"+s+"\':\n"+ex);
					mText.append("Help...failed.\nLook at console for details.\n"); return; }
				finally {
					try { fpp.close(); }
					catch(IOException ex) { System.out.println("Could not close file \'"+s+"\'.");
				}}
			}});

		m.add(mItem = new MenuItem("MIDITXT-Syntax",new MenuShortcut(89)));		// Y for sYntax
		mItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				FileInputStream fpp = null;
				String s = new String("MIDITXTSyntax.txt");
				try { fpp = new FileInputStream(new File(s)); }
				catch(IOException ex) {
					System.out.println("File \'" + s + "\' not found.");
					mText.append("Help...failed.\nLook at console for details.\n"); return; }
				try {
					s = MIDIUtil.getLine(fpp);
					while(s != null) { mText.append(s+"\n"); s = MIDIUtil.getLine(fpp); }
					fpp.close(); }
				catch(IOException ex) { System.out.println("Could not close file \'"+s+"\'."); }
				catch(MIDIException ex) {
					System.out.println("Error reading file \'"+s+"\':\n"+ex);
					mText.append("Help...failed.\nLook at console for details.\n"); return; }
				finally {
					try { fpp.close(); }
					catch(IOException ex) { System.out.println("Could not close file \'"+s+"\'.");
				}}
			}});


		mFrame.setSize(480,320);
		mFrame.show();
	}


//  --------------------------------- REMOVE DIALOG --------------------------------------


	void removeDialog()
	// opens the remove dialog
	{
		mDialog = new Dialog(mFrame,"Remove Events",true);
		mDialog.addWindowListener(new WindowListener() {
			 public void windowOpened(WindowEvent e) {}
			 public void windowClosing(WindowEvent e) { mDialog.dispose(); }
			 public void windowClosed(WindowEvent e) {}
			 public void windowIconified(WindowEvent e) {}
			 public void windowDeiconified(WindowEvent e) {}
			 public void windowActivated(WindowEvent e) {}
			 public void windowDeactivated(WindowEvent e) {}
			});
		mDialog.setLayout(new BorderLayout(2,15));

		Panel mLabelPanel = new Panel(new GridLayout(4,1));			// Label Panel
		mLabelPanel.add(new Label("Select Track:",Label.LEFT));
		mLabelPanel.add(new Label("Select Event-Type:",Label.LEFT));
		mLabelPanel.add(new Label("Select MIDI-Channel:",Label.LEFT));
		mLabelPanel.add(new Label("Select Controller:",Label.LEFT));

		Panel mChoicePanel = new Panel(new GridLayout(4,1));		// Choice Panel

		Choice mTrackChoice = new Choice();											// Select Track Choice
		mTrackChoice.add("All Tracks");
		for(int i = 0;i < main.trackVector.size(); i++) {
			int trk = ((MIDITrack)main.trackVector.elementAt(i)).trackNumber();
			mTrackChoice.add("Track "+new Integer(trk).toString()); }
		mChoicePanel.add(mTrackChoice);

		Choice mEventChoice = new Choice();											// Select EventType Choice
		mEventChoice.add("All Events");
		mEventChoice.add("Notes");
		mEventChoice.add("After Touch");
		mEventChoice.add("Channel Pressure");
		mEventChoice.add("Pitch Bend");
		mEventChoice.add("Program Change");
		mEventChoice.add("Control Change");
		mEventChoice.add("System Messages");
		mEventChoice.add("Meta Events");
		mEventChoice.addItemListener(new ItemListener() {	// inner class deactivating choices
			public void itemStateChanged(ItemEvent e) {
				Panel p = (Panel)(mDialog.getComponent(1));
				String s = ((Choice)(p.getComponent(1))).getSelectedItem();
				if(s.equals("Control Change"))														// Controller selectable
					((Choice)(p.getComponent(3))).setEnabled(true);
				else ((Choice)(p.getComponent(3))).setEnabled(false);
				if(s.equals("System Messages") || s.equals("Meta Events"))	// Channel disabled
					((Choice)(p.getComponent(2))).setEnabled(false);
				else ((Choice)(p.getComponent(2))).setEnabled(true);
			}});
		mChoicePanel.add(mEventChoice);

		Choice mChannelChoice = new Choice();										// Select MIDI-Channel Choice
		mChannelChoice.add("All Channels");
		for(int i = 1;i < 17; i++) mChannelChoice.add("Channel "+new Integer(i).toString());
		mChoicePanel.add(mChannelChoice);

		Choice mControlChoice = new Choice();										// Select Controller Choice
		mControlChoice.add("All Controllers");
		for(int i = 0;i < 128; i++)
			mControlChoice.add(new Integer(i).toString()+": "+MIDIEventUtil.controlNameFor[i]);
		mControlChoice.setEnabled(false);
		mChoicePanel.add(mControlChoice);

		Panel mActionPanel = new Panel();												// Remove, Chancel Panel
		Button mRemove = new Button("Remove");
		Button mChancel = new Button("Chancel");
		mRemove.addActionListener(new ActionListener() {	// inner class handling remove
			public void actionPerformed(ActionEvent e) {
				int trk = -1;
				int chn = -1;
				Panel p = (Panel)(mDialog.getComponent(1));
				String s = ((Choice)(p.getComponent(0))).getSelectedItem(); 	// selected Track
				if(!s.equals("All Tracks")) trk = new Integer(s.substring(6)).intValue();
				s = ((Choice)(p.getComponent(2))).getSelectedItem();					// selected Channel
				if(!s.equals("All Channels")) chn = new Integer(s.substring(8)).intValue() - 1;
				int ctr = ((Choice)(p.getComponent(3))).getSelectedIndex() - 1;	// sel Controller
				s = ((Choice)(p.getComponent(1))).getSelectedItem();					// sel EventType
				if(s.equals("Notes")) {
					main.deleteEvent(trk,-1,0x80,chn,null);
					main.deleteEvent(trk,-1,0x90,chn,null); }
				else if(s.equals("All Events"))       main.deleteEvent(trk,-1,-1,chn,null);
				else if(s.equals("After Touch"))      main.deleteEvent(trk,-1,0xa0,chn,null);
				else if(s.equals("Channel Pressure")) main.deleteEvent(trk,-1,0xd0,chn,null);
				else if(s.equals("Pitch Bend"))       main.deleteEvent(trk,-1,0xe0,chn,null);
				else if(s.equals("Program Change"))   main.deleteEvent(trk,-1,0xc0,chn,null);
				else if(s.equals("Meta Events"))      main.deleteEvent(trk,-1,0xf0,0x0f,null);
				else if(s.equals("System Messages")) {
					main.deleteEvent(trk,-1,0xf0,0x07,null);
					main.deleteEvent(trk,-1,0xf0,0x00,null); }
				else if(s.equals("Control Change")) {
					byte[] data = {(byte)ctr,-1};
					main.deleteEvent(trk,-1,0xb0,chn,data); }
				mDialog.dispose();
			}});
		mActionPanel.add(mRemove);
		mChancel.addActionListener(new ActionListener() {	// inner class chanceling remove
			public void actionPerformed(ActionEvent e) { mDialog.dispose(); }});
		mActionPanel.add(mChancel);

		mDialog.add(mLabelPanel,BorderLayout.WEST);
		mDialog.add(mChoicePanel,BorderLayout.CENTER);
		mDialog.add(mActionPanel,BorderLayout.SOUTH);
		mDialog.pack();
		mDialog.show();
	}
}